# -*- coding: utf-8 -*-
"""
Created on Wed Jun  9 12:49:47 2021

@author: hs-702
"""

import numpy as np
import pandas as pd
import matplotlib.pyplot as plt

from sklearn.model_selection import train_test_split
from sklearn.metrics import accuracy_score, precision_score, recall_score, roc_auc_score
from sklearn.metrics import f1_score, confusion_matrix, precision_recall_curve, roc_curve
from sklearn.preprocessing import StandardScaler
from sklearn.linear_model import LogisticRegression
from lightgbm import LGBMClassifier
from xgboost import XGBClassifier
from sklearn.ensemble import GradientBoostingClassifier
from sklearn.ensemble import RandomForestClassifier

diabetes_data = pd.read_csv('E:/no1/cho/archive/diabetes.csv')

test1 = diabetes_data.describe()
diabetes_data.info()

def get_clf_eval(y_test, pred=None, pred_proba=None):
    confusion = confusion_matrix( y_test, pred)
    accuracy = accuracy_score(y_test , pred)
    precision = precision_score(y_test , pred)
    recall = recall_score(y_test , pred)
    f1 = f1_score(y_test,pred)
    roc_auc = roc_auc_score(y_test, pred_proba)
    print('정확도: {0:.4f}, 정밀도: {1:.4f}, 재현율: {2:.4f},\
    F1: {3:.4f}, AUC:{4:.4f}'.format(accuracy, precision, recall, f1, roc_auc))


def get_model_predict(model, X_train, X_test, y_train, y_test):
    print('model: ' , type(model).__name__)
    model.fit(X_train, y_train)
    pred = model.predict(X_test)
    pred_proba = model.predict_proba(X_test)[:, 1]
    get_clf_eval(y_test , pred, pred_proba)


X = diabetes_data.iloc[:, :-1]
y = diabetes_data.iloc[:, -1]

X_train, X_test, y_train, y_test = train_test_split(X, y, test_size = 0.2, random_state = 156, stratify=y)

lr_clf = LogisticRegression()
LGB_clf = LGBMClassifier()
XGB_clf = XGBClassifier()
GB_clf = GradientBoostingClassifier()
rf_clf = RandomForestClassifier()

total = [lr_clf ,LGB_clf, XGB_clf, GB_clf, rf_clf]

for row in total:
    get_model_predict(row, X_train, X_test, y_train, y_test )
    
test = X['SkinThickness'].isnull().sum()

bool0 = X['SkinThickness'] == 0
bool1 = X['Pregnancies'] == 0
bool2 = X['Pregnancies'] > 0

#print(X['SkinThickness'].value_counts())

X['SkinThickness'][bool0 & bool1] = 15
X['SkinThickness'][bool0 & bool2] = 25

#print(X['SkinThickness'].value_counts())

test = X.describe()

test1 = X['BMI'].value_counts()

print(test1[36.6])
print(test1[32])
print(test1[27.3])

bool3 = X['BMI'] == 0
print(bool3.sum())

bool4 = bool3 & X['BloodPressure'] > 110
print(bool4.sum())

bool5 = bool3 & (X['BloodPressure'] <= 110) & (X['BloodPressure'] > 90)
print(bool5.sum())

bool6 = bool3 & (X['BloodPressure'] <= 90)
print(bool6.sum())

X['BMI'][bool3] = 36.6
X['BMI'][bool4] = 32
X['BMI'][bool5] = 27.3

print('-------------------------------------')

test2 = X['BMI'].value_counts()
print(test2[36.6])
print(test2[32])
print(test2[27.3])

X_train, X_test, y_train, y_test = train_test_split(X, y, test_size = 0.2, random_state = 156, stratify=y)

# 로지스틱 회귀로 학습,예측 및 평가 수행. 
lr_clf = LogisticRegression()
LGB_clf = LGBMClassifier()
XGB_clf = XGBClassifier()
GB_clf = GradientBoostingClassifier()
rf_clf = RandomForestClassifier()

total = [lr_clf ,LGB_clf, XGB_clf, GB_clf, rf_clf]

for row in total:
    get_model_predict(row, X_train, X_test, y_train, y_test )

from sklearn.preprocessing import Binarizer

def get_eval_by_threshold(y_test , pred_proba_c1, thresholds):
    # thresholds 리스트 객체내의 값을 차례로 iteration하면서 Evaluation 수행.
    for custom_threshold in thresholds:
        binarizer = Binarizer(threshold=custom_threshold).fit(pred_proba_c1) 
        custom_predict = binarizer.transform(pred_proba_c1)
        print('임곗값:',custom_threshold)
        get_clf_eval(y_test , custom_predict, pred_proba_c1)

print('-------------------------------------------------------------')
thresholds = [0.3 , 0.33 ,0.36,0.39, 0.42 , 0.45 ,0.48, 0.50]
pred_proba = lr_clf.predict_proba(X_test)
get_eval_by_threshold(y_test, pred_proba[:,1].reshape(-1,1), thresholds)













